/*
 *   Copyright (c) 2003 David Parsons. All rights reserved.
 *   
 *   Redistribution and use in source and binary forms, with or without
 *   modification, are permitted provided that the following conditions
 *   are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *  3. All advertising materials mentioning features or use of this
 *     software must display the following acknowledgement:
 *     
 *   This product includes software developed by David Parsons
 *   (orc@pell.chi.il.us)
 *
 *  4. My name may not be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *     
 *  THIS SOFTWARE IS PROVIDED BY DAVID PARSONS ``AS IS'' AND ANY
 *  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 *  PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL DAVID
 *  PARSONS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 *  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * manipulate virtual domain password files (vm-pop3d, et al)
 *
 */

#include "vpw.h"

#if defined(__FreeBSD__) || defined(__OpenBSD__)
# include <stdlib.h>		// freebsd has copied malloc.h to stdlib.h
#else
# include <malloc.h>
#endif
#include <fcntl.h>
#include <errno.h>
#include <string.h>

extern int getrecord(FILE *f, ...);

static FILE *vp_file = 0;

int
setvpw(char *f)
{
    if (vp_file) {
	errno = EEXIST;
	return 0;
    }
    return (vp_file = fopen(f, "r")) != 0;
}


void
setvpwfile(FILE* f)
{
    vp_file = f;
}


void
endvpw()
{
    fclose(vp_file);
    vp_file = 0;
}


/*
 * get the next virtual password file entry
 */
struct vpwent *
getvpwent()
{
    static struct vpwent vpw;
    int rc;

    memset(&vpw, 0, sizeof vpw);
    while ( (rc=getrecord(vp_file, &vpw.v_name, &vpw.v_pwd, 0)) >= 0)
	if (rc >= 2)	/* ignore short records */
	    return &vpw;
    return 0;
} /* getvpent */


/*
 * write out a virtual password entry
 */
int
putvpwent(FILE *dest, struct vpwent *user)
{
    if (dest) {
	fprintf(dest, "%s:%s\n", user->v_name, user->v_pwd);
	return ferror(dest) ? -1 : 0;
    }
    return -1;
}


/*
 * get a virtual password entry by name
 */
struct vpwent *
getvpwnam(char *name)
{
    struct vpwent *ret;

    if (vp_file) {
	rewind(vp_file);
	while (ret = getvpwent())
	    if (strcasecmp(ret->v_name, name) == 0)
		return ret;
    }
    return 0;
} /* getvpwnam */
